// Learn TypeScript:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/typescript.html
//  - [English] http://www.cocos2d-x.org/docs/creator/manual/en/scripting/typescript.html
// Learn Attribute:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/reference/attributes.html
//  - [English] http://www.cocos2d-x.org/docs/creator/manual/en/scripting/reference/attributes.html
// Learn life-cycle callbacks:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/life-cycle-callbacks.html
//  - [English] http://www.cocos2d-x.org/docs/creator/manual/en/scripting/life-cycle-callbacks.html

const {ccclass, property} = cc._decorator;

@ccclass
export default class NewClass extends cc.Component {
    // LIFE-CYCLE CALLBACKS:

    // onLoad () {}

    @property(cc.Node)
    btnClear:cc.Node = null

    touches: cc.Vec2[] = []
    graphics: cc.Graphics = null;

    start () {
        var canvas = cc.find('Canvas');
        canvas.on(cc.Node.EventType.MOUSE_DOWN, this.onTouchStart, this);
        canvas.on(cc.Node.EventType.MOUSE_MOVE, this.onTouchMove, this);
        canvas.on(cc.Node.EventType.MOUSE_UP, this.onTouchEnd, this);

        this.graphics = this.getComponent(cc.Graphics);
        // this.graphics.moveTo(0, 0);
        // this.graphics.lineTo(0.1, 0);
        // this.graphics.lineTo(-0.11, 0);
        // this.graphics.stroke();
        console.log(cc.sys.os)
    }

    onEnable(){
        this.btnClear.on(cc.Node.EventType.TOUCH_END, this.onClear, this);
    }

    onDisable(){
        this.btnClear.off(cc.Node.EventType.TOUCH_END, this.onClear, this);
    }

    onTouchStart (event:cc.Event.EventTouch) {
        this.touches.length = 0;
        this.touches.push(event.getLocation());
        console.log("start draw")
    }

    onTouchMove (event:cc.Event.EventMouse) {
        console.log("move draw")
        let touches = this.touches;
        touches.push(event.getLocation());

        const MIN_POINT_DISTANCE = 2;

        // this.graphics.clear();
        let worldPos = this.node.convertToWorldSpaceAR(cc.v2());
        this.graphics.moveTo(touches[0].x - worldPos.x, touches[0].y - worldPos.y);
        let lastIndex = 0;
        for (let i = 1, l = touches.length; i < l; i++) {
            if (touches[i].sub(touches[lastIndex]).mag() < MIN_POINT_DISTANCE) {
                continue;
            }
            lastIndex = i;
            this.graphics.lineTo(touches[i].x - worldPos.x, touches[i].y - worldPos.y);
        }
        this.graphics.stroke();
    }

    onTouchEnd (event:cc.Event.EventTouch) {
        console.log("start end")
    }

    onClear(){
        console.log("clear")
        this.graphics.clear()
    }
}
